from lumapps.api import ApiClient


class TokenClient(ApiClient):
    def __init__(self, customer_id, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.customer_id = customer_id

    def get_token_and_expiry(self, email):
        resp = self.get_call("user/getToken", customerId=self.customer_id, email=email)
        return resp["accessToken"], int(resp["expiresAt"])

    def get_token_getter(self, email):
        def f():
            return self.get_token_and_expiry(email)

        return f



from httpx import HTTPError


def get_http_err_content(e: HTTPError) -> str:
    try:
        return e.response.content.decode()  # type: ignore
    except AttributeError:
        return ""


class ApiClientError(Exception):
    """ Base error of the ApiClient """


class ApiCallError(ApiClientError):
    """ Retrocomptiblity """


class ApiClientRequestError(ApiClientError):
    """ Error during a request with the ApiClient """



import warnings
from contextlib import AbstractContextManager
from functools import lru_cache
from json import dumps, loads
from pathlib import Path
from textwrap import TextWrapper
from time import time
from typing import (
    IO,
    Any,
    Callable,
    Dict,
    Generator,
    List,
    Optional,
    Sequence,
    Tuple,
    Union,
)

from httpx import Client

# from authlib.integrations.requests_client import OAuth2Session, AssertionSession
from lumapps.api.authlib_helpers import AssertionClient, OAuth2Client
from lumapps.api.errors import ApiCallError, ApiClientError
from lumapps.api.utils import (
    FILTERS,
    GOOGLE_APIS,
    _parse_endpoint_parts,
    get_discovery_cache,
    get_endpoints,
    method_from_discovery,
    pop_matches,
)

LUMAPPS_SCOPE = ["https://www.googleapis.com/auth/userinfo.email"]
LUMAPPS_VERSION = "v1"
LUMAPPS_NAME = "lumsites"
LUMAPPS_BASE_URL = "https://lumsites.appspot.com"
FileContent = Union[IO[str], IO[bytes], str, bytes]


class ApiClient(AbstractContextManager):
    def __init__(
        self,
        auth_info: Optional[Dict[str, Any]] = None,
        api_info: Optional[Dict[str, Any]] = None,
        user: Optional[str] = None,
        token: Optional[str] = None,
        token_getter: Optional[Callable[[], Tuple[str, int]]] = None,
        prune: bool = False,
        no_verify: bool = False,
        proxy_info: Optional[Dict[str, Any]] = None,
    ):
        """
            Args:
                auth_info: When specified, a service account or a web auth JSON dict.
                api_info: When specified, a JSON dict containing the description of your
                    api. Defaults to LumApps API.
                user: Email of user on behalf of whom to authenticate using domain-wide
                    delegation.
                token: A bearer access token.
                token_getter: A bearer access token getter function.
                prune: Whether or not to use FILTERS to prune LumApps API responses.
                no_verify: Disables SSL verification.
                proxy_info: When specified, a JSON dict with proxy parameters.
        """
        self._token_expiry = 0
        self.no_verify = no_verify
        self.proxy_info = proxy_info
        self.prune = prune
        self._auth_info = auth_info
        self._token = None
        self._endpoints = None
        self._client = None
        self._headers: dict = {}
        if api_info is None:
            api_info = {}
        api_info.setdefault("name", LUMAPPS_NAME)
        api_info.setdefault("version", LUMAPPS_VERSION)
        api_info.setdefault("base_url", LUMAPPS_BASE_URL)
        api_info.setdefault("scopes", LUMAPPS_SCOPE)
        self.api_info = api_info
        api_name = api_info["name"]
        self._scope = " ".join(api_info["scopes"])
        api_ver = api_info["version"]
        prefix = "" if api_name in GOOGLE_APIS else "/_ah/api"
        self._api_url = f"{prefix}/{api_name}/{api_ver}"
        self._discovery_url = (
            f"{self.base_url}{prefix}/discovery/v1/apis/{api_name}/{api_ver}/rest"
        )
        self.token_getter = token_getter
        self.user = user
        self.token = token
        self.cursor = None

    def __exit__(self, *exc):
        self.close()
        return False

    def close(self):
        if self._client:
            self._client.close()
            self._client = None

    @property
    def base_url(self):
        return self.api_info["base_url"].rstrip("/")

    @property
    def token(self):
        return self._token

    @token.setter
    def token(self, v):
        if self._token and self._token == v:
            return
        self._token = v
        self._headers["authorization"] = f"Bearer {self._token}"
        if self._client:
            self._client.headers.update(self._headers)

    def _create_client(self):
        auth = self._auth_info
        kwargs = {
            "base_url": self.base_url,
            "headers": self._headers,
            "verify": not self.no_verify,
            "timeout": 120,
        }
        if self.proxy_info:
            scheme = self.proxy_info.get("scheme", "https")
            host = self.proxy_info["host"]
            port = self.proxy_info["port"]
            user = self.proxy_info.get("user") or ""
            pwd = self.proxy_info.get("password") or ""
            if user or pwd:
                proxy = f"{scheme}://{user}:{pwd}@{host}:{port}"
            else:
                proxy = f"{scheme}://{host}:{port}"
            kwargs["proxies"] = {"https": proxy, "http": proxy}
        else:
            kwargs["proxies"] = None
        if not auth and self._token:
            s = Client(**kwargs)
        elif auth and "refresh_token" in auth:
            s = OAuth2Client(
                client_id=auth["client_id"],
                client_secret=auth["client_secret"],
                scope=self._scope,
                **kwargs,
            )
            s.refresh_token(auth["token_uri"], refresh_token=auth["refresh_token"])
        elif auth:  # service account
            claims = {"scope": self._scope} if self._scope else {}
            s = AssertionClient(
                token_endpoint=auth["token_uri"],
                issuer=auth["client_email"],
                audience=auth["token_uri"],
                claims=claims,
                subject=self.user,
                key=auth["private_key"],
                header={"alg": "RS256"},
                **kwargs,
            )
        else:
            raise ApiClientError(
                "No authentication provided (token_getter, auth_info, or token)."
            )
        return s

    @property
    def session(self) -> Client:
        warnings.warn("Use client instead", DeprecationWarning, stacklevel=2)
        return self.client

    @property
    def client(self) -> Client:
        """Setup the client object."""
        self._check_access_token()
        if self._client is None:
            self._client = self._create_client()
        return self._client

    @property  # type: ignore
    @lru_cache()
    def discovery_doc(self):
        url = self._discovery_url
        d = get_discovery_cache().get(url)
        if d and isinstance(d, str):
            return loads(d)
        elif d and isinstance(d, dict):
            return d
        else:
            resp = self.client.get(url)
            resp_doc = resp.json()
            get_discovery_cache().set(url, resp_doc)
            return resp_doc

    def _check_access_token(self):
        if not self.token_getter:
            return
        exp = self._token_expiry
        if exp and exp > time() + 120:
            return
        self.token, self._token_expiry = self.token_getter()

    def _prune(self, name_parts, content):
        """Prune the api response.
        """
        if not self.prune:
            return content
        for ep_filter in FILTERS:
            ep_filter_parts = ep_filter.split("/")
            if len(name_parts) != len(ep_filter_parts):
                continue
            for filter_part, part in zip(ep_filter_parts, name_parts):
                if filter_part not in ("*", part):
                    break
            else:
                for pth in FILTERS[ep_filter]:
                    if isinstance(content, list):
                        for o in content:
                            pop_matches(pth, o)
                    else:
                        pop_matches(pth, content)
        return content

    def get_new_client_as_using_dwd(self, user_email: str) -> "ApiClient":
        """ Get a new ApiClient using domain-wide delegation """
        return ApiClient(
            auth_info=self._auth_info,
            api_info=self.api_info,
            user=user_email,
            no_verify=self.no_verify,
            proxy_info=self.proxy_info,
            prune=self.prune,
        )

    def get_new_client_as(
        self, user_email: str, customer_id: Optional[str] = None
    ) -> "ApiClient":
        """ Get a new ApiClient using an authorized client account by obtaining a
            token using the user/getToken endpoint.

            Args:
                user_email (str): User you want to authenticate on behalf of
                customer_id (str): Id of the LumApps customer the user belong to

            Returns:
                ApiClient: A new instance of the ApiClient correctly authenticated.
        """
        client = ApiClient(
            auth_info=self._auth_info,
            api_info=self.api_info,
            no_verify=self.no_verify,
            proxy_info=self.proxy_info,
            prune=self.prune,
        )
        token_infos: Any = client.get_call(
            "user/getToken", customerId=customer_id, email=user_email
        )
        token = token_infos["accessToken"]
        return ApiClient(
            api_info=self.api_info,
            token=token,
            user=user_email,
            no_verify=self.no_verify,
            proxy_info=self.proxy_info,
            prune=self.prune,
        )

    @property
    def endpoints(self):
        if self._endpoints is None:
            self._endpoints = get_endpoints(self.discovery_doc)
        return self._endpoints

    def get_help(self, name_parts, debug=False):
        help_lines = []

        def add_line(li):
            help_lines.append(li)

        wrapper = TextWrapper(initial_indent="\t", subsequent_indent="\t")
        ep_info = self.endpoints[name_parts]
        method = ep_info.get("httpMethod", "?")
        add_line(f"{method} endpoint: " + " ".join(name_parts) + "\n")
        if "description" in ep_info:
            add_line(ep_info["description"].strip() + "\n")
        if debug:
            add_line(dumps(ep_info, indent=4, sort_keys=True))
        params = ep_info.get("parameters", {})
        if method == "POST":
            params.update(
                {
                    "body": {"required": True, "type": "JSON"},
                    "fields": {"type": "JSON"},
                }
            )
        if not params:
            add_line("Endpoint takes no parameters")
        else:
            add_line("Parameters (*required):")
            for param_name in sorted(params):
                param = params[param_name]
                descr = param.get("description")
                descr = "\n" + wrapper.fill(descr) if descr else ""
                add_line(
                    "  {}: {} {} {}".format(
                        param_name,
                        param["type"],
                        "*" if param.get("required") else "",
                        descr,
                    )
                )
        return "\n".join(help_lines)

    def get_endpoints_info(self, endpoints):
        lines = []
        for name_parts in endpoints:
            descr = self.endpoints[name_parts].get("description", "")
            lines.append((" ".join(name_parts), descr.strip().split("\n")[0]))
        longest_name = max(len(li1[0]) for li1 in lines)
        fmt = "  {{: <{}}}  {{}}".format(longest_name)
        return "\n".join(fmt.format(*li2) for li2 in lines)

    def _expand_path(self, path, endpoint: dict, params: dict):
        param_specs = endpoint.get("parameters", {})
        path_args: dict = {}
        for param, specs in param_specs.items():
            if specs.get("required") is True and param not in params:
                raise ApiCallError(f"Missing required parameter {param}")
            if specs["location"] == "path":
                path_args[param] = None
        if path_args:
            for param in path_args:
                path_args[param] = params.pop(param)
            path = path.format(**path_args)
        return path

    @lru_cache()
    def _get_endpoint_and_path_from_discovery(self, name_parts):
        endpoint = method_from_discovery(self.discovery_doc, name_parts)
        if not endpoint:
            raise ApiCallError(f"Endpoint {'.'.join(name_parts)} not found")
        if endpoint.get("mediaUpload"):
            raise ApiCallError(
                f"Endpoint {'.'.join(name_parts)} is for uploads, "
                f"use upload_call method instead of get_call or iter_call"
            )
        path = self._api_url + "/" + endpoint.get("path") or "/".join(name_parts)
        return endpoint, path

    def _get_verb_path_params(self, name_parts, params: dict):
        endpoint, path = self._get_endpoint_and_path_from_discovery(tuple(name_parts))
        path = self._expand_path(path, endpoint, params)
        verb = endpoint.get("httpMethod")
        return verb, path, params

    def _call(self, name_parts: Sequence[str], params: dict, json=None):
        """ Construct the call """
        verb, path, params = self._get_verb_path_params(name_parts, params)
        resp = self.client.request(verb, path, params=params, json=json)
        resp.raise_for_status()
        if not resp.content:
            return None
        return resp.json()

    @staticmethod
    def _pop_body(params: dict):
        body = params.pop("body", None)
        body = loads(body) if isinstance(body, str) else body
        return body

    def upload(self, file_content: FileContent, metadata: dict, *name_parts, **params):
        name_parts = _parse_endpoint_parts(name_parts)
        endpoint: Any = method_from_discovery(self.discovery_doc, name_parts)  # type: ignore  # noqa
        if not endpoint.get("mediaUpload"):
            raise ApiCallError(
                f"Endpoint {'.'.join(name_parts)} is not for uploads, "
                f"use get_call or iter_call instead."
            )
        verb = endpoint.get("httpMethod")
        upload_specs = endpoint["mediaUpload"]["protocols"]["simple"]
        path: Any = self.discovery_doc["rootUrl"].rstrip("/") + upload_specs["path"]  # type: ignore  # noqa
        path = self._expand_path(path, endpoint, params)
        files = {
            "data": ("metadata", dumps(metadata), "application/json; charset=UTF-8"),
            "file": file_content,
        }
        resp = self.client.request(verb, path, params=params, files=files)
        resp.raise_for_status()
        return resp.json()

    def upload_call(
        self, fpath: Union[Path, str], metadata: dict, *name_parts, **params
    ):
        with Path(fpath).open("rb") as fh:
            return self.upload(fh, metadata, *name_parts, **params)

    def get_call(
        self, *name_parts, **params
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], None]:
        """Generic function to call a lumapps endpoint

            Args:
                *name_parts: Endpoint, eg user/get or "user", "get"
                **params: Parameters of the call

            Returns:
                Object or objects returned by the endpoint call.

            Example:
                List feedtypes in LumApps:
                -> GET https://.../_ah/api/lumsites/v1/feedtype/list

                With this endpoint:

                    >>> feedtypes = get_call("feedtype/list")
                    >>> print(feedtypes)
        """
        name_parts = _parse_endpoint_parts(name_parts)
        items: List[dict] = []
        self.cursor = cursor = params.pop("cursor", None)
        body = self._pop_body(params)
        while True:
            if cursor:
                if body is not None:
                    body["cursor"] = cursor
                else:
                    params["cursor"] = cursor
            response = self._call(name_parts, params, body)
            if response is None:
                return None

            more = response.get("more")
            response_items = response.get("items")
            if more:
                if response_items:
                    self.cursor = cursor = response["cursor"]
                    items.extend(response_items)
                else:
                    # No results but a more field set to true ...
                    # ie, the api return something wrong
                    self.cursor = cursor = None
                    return self._prune(name_parts, items)
            else:
                # No more result to get
                if response_items:
                    self.cursor = cursor = None
                    items.extend(response_items)
                    return self._prune(name_parts, items)
                else:
                    # No results, return
                    self.cursor = cursor = None
                    # special case of
                    return [] if more is False else self._prune(name_parts, response)

    def iter_call(
        self, *name_parts, **params
    ) -> Generator[
        Union[Dict[str, Any], List[Dict[str, Any]]],
        Union[Dict[str, Any], List[Dict[str, Any]]],
        None,
    ]:
        """
            Args:
                *name_parts: Endpoint, eg user/get or "user", "get"
                **params: Parameters of the call

            Yields:
                Objects returned by the endpoint call


            Example:
                List feedtypes in LumApps:
                -> GET https://.../_ah/api/lumsites/v1/feedtype/list

                With this endpoint:

                    >>> feedtypes = iter_call("feedtype/list")
                    >>> for feedtype in feedtypes: print(feedtype)
        """
        name_parts = _parse_endpoint_parts(name_parts)
        self.cursor = cursor = params.pop("cursor", None)
        body = self._pop_body(params)
        while True:
            if cursor:
                if body is not None:
                    body["cursor"] = cursor
                else:
                    params["cursor"] = cursor

            response = self._call(name_parts, params, body)
            more = response.get("more")
            items = response.get("items")

            if more:
                if items:
                    # Yield the results and continue the loop
                    self.cursor = cursor = response["cursor"]
                    for item in items:
                        yield self._prune(name_parts, item)
                else:
                    # No results but a more field set to true ...
                    # ie, the api return something wrong
                    self.cursor = cursor = None
                    return
            else:
                # No more result to get
                if items:
                    # Yield the last results and then return
                    self.cursor = cursor = None
                    for item in items:
                        yield self._prune(name_parts, item)
                    else:
                        return
                else:
                    # No results, return
                    self.cursor = cursor = None
                    return

    def get_matching_endpoints(self, name_parts):
        # find exact matches of all parts up to but excluding last
        matches = [n for n in self.endpoints if len(n) >= len(name_parts)]
        for i, part in enumerate(name_parts[:-1]):
            matches = [n for n in matches if len(n) >= i and n[i] == part]
        # find 'startswith' matches of the last part
        last = name_parts[-1]
        idx = len(name_parts) - 1
        matches = [m for m in matches if len(m) >= idx and m[idx].startswith(last)]
        if not matches:
            return "Endpoint not found"
        return (
            "Endpoint not found. Did you mean one of these?\n"
            + self.get_endpoints_info(sorted(matches))
        )



from typing import Any, Dict, Optional, Union

import httpx

from lumapps.api.client import ApiClient
from lumapps.api.errors import ApiClientError


def create_new_media(
    client: ApiClient,
    file_data_or_path: Union[bytes, str],
    doc_path: str,
    filename: str,
    mimetype: str,
    is_shared: bool,
    lang: Optional[str] = "en",
) -> Optional[Dict[str, Any]]:
    """ Upload a file and create a new media in LumApps media library.

        Args:
            client: The ApiClient used to make httpx to the LumApps Api.
            file_data_or_path: The filepath (str) or the data (bytes) to upload.
            doc_path: The doc path of the media to upload, this will decide where the media will go in your media library
                            (eg: provider=`<`my_provider`>`/site=`<`my_site_id`>`/resource=`<`my_parent_folder_id`>`)
            filename: The name of the file to upload. Once uploaded the file will appear with that name.
            mimetype: The mimeType fo the file to upload.
            is_shared: Wether the file is shared or not. Non shared files will only be visible by you.
            lang: The lang of the file to upload (default: "en").

        Raises:
            Exception: The data or file path type provided is not supported.

        Returns:
            Return the uploaded media if successfull, None otherwise.
    """  # noqa

    if isinstance(file_data_or_path, str):
        file_data = open(file_data_or_path, "rb")
    elif isinstance(file_data_or_path, bytes):
        file_data = file_data_or_path  # type: ignore
    else:
        raise ApiClientError(
            "File data or path type not supported: {}".format(type(file_data_or_path))
        )

    # Get upload url for the document
    body = {
        "fileName": filename,
        "lang": lang,
        "parentPath": doc_path,
        "shared": is_shared,
        "success": "/upload",
    }
    upload_infos: Any = client.get_call("document/uploadUrl/get", body=body)
    upload_url = upload_infos["uploadUrl"]

    # Upload
    files_tuple_list: Any = [("files", (filename, file_data, mimetype))]
    response = httpx.post(
        upload_url,
        headers={"Authorization": "Bearer " + client.token},
        files=files_tuple_list,  # type: ignore
    )
    doc = response.json().get("items")

    if doc:
        return doc[0]
    return None


def add_media_file_for_lang(
    client: ApiClient,
    media: Dict[str, Any],
    file_data_or_path: str,
    filename: str,
    mimetype: str,
    lang: Optional[str] = "en",
    croppedContent: Optional[bool] = False,
) -> Optional[Dict[str, Any]]:
    """ Add a file to an existing LumApps media.

        Args:
            client: The ApiClient used to make httpx to the LumApps Api.
            media: The LumApps media on which the files have to be uploaded.
            file_data_or_path (Union[bytes, str]): The filepath (str) or the data (bytes) to upload.
            filename: The name of the file to upload. Once uploaded the file will appear with that name.
            mimetype: The mimeType fo the file to upload.
            lang: The lang of the file to upload (default: "en").
            croppedContent (bool): Wether to add the file to the croppedContent instead or content (default: False)

        Returns:
            The updated media if succesfull, otherwise None.
    """  # noqa

    # upload the file
    uploaded_file = _upload_new_media_file_of_given_lang(
        client=client,
        file_data_or_path=file_data_or_path,
        filename=filename,
        mimetype=mimetype,
        lang=lang,
    )
    if not uploaded_file:
        return media

    # update the media
    where = "croppedContent" if croppedContent else "content"
    media[where].append(uploaded_file)
    saved: Any = client.get_call("document/update", body=media)
    return saved


def _upload_new_media_file_of_given_lang(
    client: ApiClient,
    file_data_or_path: Union[bytes, str],
    filename: str,
    mimetype: str,
    lang: Optional[str] = "en",
    prepare_for_lumapps: Optional[bool] = False,
):
    """ Upload a file to lumapps without creating an associated media

        Args:
            client: The ApiClient used to make httpx to the LumApps Api.
            file_data_or_path: The filepath or the data to upload.
            filename: The name of the file to upload. Once uploaded the file will appear with that name.
            mimetype: The mimeType fo the file to upload.
            lang: The lang of the file to upload (default: "en").
            prepare_for_lumapps: prepare the return uploaded file to be usable as a lumapps media object (default: False)

        Raises:
            Exception: The data or file path type provided is not supported.

        Returns:
            Return the uploaded file if successfull, None otherwise.
        
        Notes:
            This is intended to be used to add new lang version/croppedContent to
            a LumApps media.
    """  # noqa

    if isinstance(file_data_or_path, str):
        file_data = open(file_data_or_path, "rb")
    elif isinstance(file_data_or_path, bytes):
        file_data = file_data_or_path  # type: ignore
    else:
        raise ApiClientError(
            "File data or path type not supported: {}".format(type(file_data_or_path))
        )

    # Get an upload url for the file
    response = httpx.get(
        "{}/upload?success=/upload".format(client.base_url),
        headers={"Authorization": "Bearer " + client.token},
    )
    upload_url = response.json().get("uploadUrl")

    if not upload_url:
        return

    # Upload the file
    response = httpx.post(
        upload_url,
        headers={"Authorization": "Bearer " + client.token},
        files={"upload-file": (filename, file_data, mimetype)},
    )
    uploaded_file = response.json()

    if not uploaded_file:
        return

    if prepare_for_lumapps:
        # Transform uploaded file to be accepted as a file in a media
        uploaded_file["lang"] = lang
        uploaded_file["value"] = uploaded_file["blobKey"]
        uploaded_file["servingUrl"] = uploaded_file["url"]
        uploaded_file["downloadUrl"] = uploaded_file["url"]
        uploaded_file.pop("blobKey", None)
        uploaded_file.pop("upload", None)
        uploaded_file.pop("fileLink", None)

    return uploaded_file



from lumapps.api.helpers.widgets import (  # noqa
    content_is_community,
    content_is_template,
    copy_with_new_lumapps_uuids,
    find_all_widgets,
    find_all_widgets_and_containers,
    find_one_with_key_value,
    find_widget,
    find_widget_and_container,
    iter_with_key,
    iter_with_key_value,
    new_lumapps_uuid,
    replace_key_val,
    replace_matching_key_val,
    set_new_lumapps_uuids,
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lumapps', 'lumapps.api', 'lumapps.api.helpers']

package_data = \
{'': ['*']}

install_requires = \
['Authlib>=0.14.3,<0.15.0', 'httpx>=0.13.3,<0.14.0']

entry_points = \
{'console_scripts': ['lac = lumapps.api.cli:main']}

setup_kwargs = {
    'name': 'lumapps-sdk',
    'version': '1.0.9',
    'description': '',
    'long_description': '# Lumapps SDK\n\n<p align="center">\n    <a href="https://github.com/lumapps/lumapps-sdk/actions?query=workflow%3ACI"><img alt="Action Status" src="https://github.com/lumapps/lumapps-sdk/workflows/CI/badge.svg"></a>\n    <a href="https://pypi.org/project/lumapps-sdk/"><img alt="Pypi" src="https://img.shields.io/pypi/v/lumapps-sdk"></a>\n    <a href="https://codecov.io/gh/lumapps/lumapps-sdk/branch/master"><img alt="Coverage" src="https://codecov.io/gh/lumapps/lumapps-sdk/branch/master/graph/badge.svg"></a>\n    <a href="https://github.com/ambv/black"><img alt="Black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n    <a href="#"><img alt="Black" src="https://img.shields.io/badge/python-3.6%7C3.7%7C3.8-blue"></a>\n</p>\n\n\nLumApps SDK is a set of tools to manipulate the [LumApps API](https://api.lumapps.com/docs/start)\n\nThis includes:\n\n- A client that support all the routes of the API (located in folder lumapps)\n- A set of helper classes to easily manipulate LumApps elements as Python Objects and classes (folder lumapps/helpers)\n\n\n## Installation\n\n\n```bash\npip install lumapps-sdk\n```\n\n## Getting started\n\n\n1. Get your token\n\n    LumApps supports multiple ways of authentication.\n    The fastest one to implement is the following:\n\n    Get your token by logging to your LumApps account.\n    Go to [https://sites.lumapps.com](https://sites.lumapps.com) and authenticate.\n    Once connected, open the javascript console of your browser and run:\n\n    ```javascript\n    var instance = window.location.pathname.split(\'/\');\n    instance = instance[1] == "a" ? instance[3] : instance[1]\n    fetch(window.location.origin+"/service/init?customerHost="+window.location.host+"&instanceSlug="+instance+"&    slug=").then(data=>{return data.json()}).then(res => {console.log(res.token)})\n    ```\n\n    This will generate your personal LumApps token that will be active for 60 minutes, and that we will use in the following steps\n\n2. Authenticate\n\n    ```python\n    from lumapps.api import ApiClient\n    \n    token = "MY TOKEN"\n    client = ApiClient(token=token)\n    ```\n\n3. Make your first API call\n\n    Let\'s display the full name of a registered user in lumapps\n\n    ```python\n    user_email = "YOUR EMAIL"\n    usr = api.get_call("user/get", email=user_email)\n    print("Hello {}".format(usr["fullName"]))\n    ```\n\n## Documentation\n\nThe SDK documentation is available [here](https://lumapps.github.io/lumapps-sdk/).\n\n## Code convention\n\nDocstring in PEP 484 type annotations format adapted to python 2.7 using comments.\n\n## How to get help, contribute, or provide feedback\n\nPlease refer to our [contributing guidelines](CONTRIBUTING.md).\n\n## Copyright and license\n\n\nLumApps SDK is released under the [MIT license](LICENSE.md).\n\n\n',
    'author': 'Aurélien Dentan',
    'author_email': 'aurelien@lumapps.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lumapps/lumapps-sdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.4,<4.0.0',
}


setup(**setup_kwargs)
